require "import"
import "android.widget.*"
import "android.view.*"
import "android.text.*"
import "android.content.*"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.app.AlertDialog"
import "android.app.SearchManager"  -- Import SearchManager for Google search functionality

-- Activity title
activity.setTitle("Text Content")

-- Helper function to show toast messages
function showToast(message)
    Toast.makeText(activity, message, Toast.LENGTH_SHORT).show()
end

-- Speech recognition function
local function startSpeechRecognition(language)
    local recordIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recordIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recordIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, language)

    local speechRecord = SpeechRecognizer.createSpeechRecognizer(activity)
    speechRecord.setRecognitionListener{
        onResults = function(results)
            local spokenText = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION).get(0)
            if textEditor then
                textEditor.append(spokenText .. " ")
            end
            speechRecord.destroy()
        end,
        onError = function()
            speechRecord.destroy()
            showToast("Error: Unable to recognize speech!")
        end
    }
    speechRecord.startListening(recordIntent)
end

-- Text analysis function
local function showTextAnalysis()
    local title = titleEditor.getText().toString()
    local text = textEditor.getText().toString()

    -- Calculate metrics
    local characterCount = string.len(title) + string.len(text)
    local wordCount = 0
    for _ in string.gmatch(title .. " " .. text, "%S+") do
        wordCount = wordCount + 1
    end

    local sentenceCount = 0
    for _ in string.gmatch(text, "[^%.!?]+[%.!?]") do
        sentenceCount = sentenceCount + 1
    end

    local lineCount = 0
    for _ in string.gmatch(text, "[^\n]*\n?") do
        lineCount = lineCount + 1
    end

    local paragraphCount = 0
    for paragraph in string.gmatch(text, "[^\n]+") do
        if string.match(paragraph, "%S") then
            paragraphCount = paragraphCount + 1
        end
    end

    -- Create list of results
    local results = {
        "Characters: " .. characterCount,
        "Words: " .. wordCount,
        "Sentences: " .. sentenceCount,
        "Lines: " .. lineCount,
        "Paragraphs: " .. paragraphCount
    }

    -- Create ListView
    local listView = ListView(activity)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, String(results))
    listView.setAdapter(adapter)

    -- Show in a new dialog
    local builder = AlertDialog.Builder(activity)
    builder.setTitle("Text Analysis")
    builder.setView(listView)
    builder.setPositiveButton("OK", nil)
    local dialog = builder.create()
    dialog.show()
end

-- Define a table to store saved texts
local savedTexts = {}

-- Function to display all saved texts
local function showSavedTexts()
    if #savedTexts == 0 then
        showToast("No saved texts available!")
        return
    end

    -- Convert savedTexts table to a string array for ListView
    local adapterData = String(savedTexts)

    -- Create ListView to display saved texts
    local listView = ListView(activity)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, adapterData)
    listView.setAdapter(adapter)

    -- Create a dialog to show the ListView
    local builder = AlertDialog.Builder(activity)
    builder.setTitle("Saved Texts")
    builder.setView(listView)
    builder.setPositiveButton("OK", nil)
    local dialog = builder.create()
    dialog.show()
end

-- Define main layout with title EditText added
mainLayout = {
   LinearLayout,
   orientation = "vertical",
   layout_width = "fill",
   layout_height = "fill",
   
   {
       EditText,
       id = "titleEditor",
       layout_width = "match_parent",
       layout_height = "wrap_content",
       hint = "Enter Title Here...",
       textSize = "16sp",
       inputType = InputType.TYPE_CLASS_TEXT,
       gravity = "top",
       layout_marginBottom = "8dp"
   },
   
   {
       EditText,
       id = "textEditor",
       layout_width = "match_parent",
       layout_height = "0dp",
       layout_weight = 1,
       hint = "Write your text here...",
       textSize = "16sp",
       inputType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_MULTI_LINE,
       gravity = "top",
   },
   
   {
       LinearLayout,
       orientation = "horizontal",
       layout_width = "match_parent",
       layout_height = "wrap_content",
       gravity = "center",

       {
           Button,
           id="saveButton",
           layout_width="0dp", 
           layout_height="wrap_content", 
           layout_weight=1, 
           text="Save Text"
      },
      
      {
           Button,
           id="shareButton",
           layout_width="0dp", 
           layout_height="wrap_content", 
           layout_weight=1, 
           text="Share Text"
      },
      
      {
           Button,
           id="copyButton",
           layout_width="0dp", 
           layout_height="wrap_content", 
           layout_weight=1, 
           text="Copy Text"
      },
      
      {
           Button,
           id="pasteButton",
           layout_width="0dp", 
           layout_height="wrap_content", 
           layout_weight=1, 
           text="Paste Text"
      },
      
      {
           Button,
           id="cutButton",
           layout_width="0dp", 
           layout_height="wrap_content", 
           layout_weight=1, 
           text="Cut Text"
      },
   },
   
   {
       LinearLayout,
       orientation = "horizontal",
       layout_width = "match_parent",
       layout_height = "wrap_content",
       gravity = "center",

       {
          Button,
          id="clearButton", 
          layout_width="0dp", 
          layout_height="wrap_content", 
          layout_weight=1, 
          text="Clear Text"
      },
      {
          Button,
          id="hindiSpeechButton", 
          layout_width="0dp", 
          layout_height="wrap_content", 
          layout_weight=1, 
          text="Hindi Speech Typing"
      },
      {
          Button,
          id="englishSpeechButton", 
          layout_width="0dp", 
          layout_height="wrap_content", 
          layout_weight=1, 
          text="English Speech Typing"
      },
      {
          Button,
          id="analyzeTextButton", 
          layout_width="0dp", 
          layout_height="wrap_content", 
          layout_weight=1, 
          text="Analyze Text"
      },
      {
          Button,
          id="googleSearchButton",  -- New button for Google Search
          layout_width="0dp", 
          layout_height="wrap_content", 
          layout_weight=1, 
          text="Google Search"  -- Button label
      }
   },

   {
      Button,
      id="showSavedTextsButton", 
      layout_width="match_parent", 
      layout_height="wrap_content", 
      text="Show Saved Texts"
   }
}

-- Set content view after defining the mainLayout
activity.setContentView(loadlayout(mainLayout))

-- Button functionalities (modified where necessary)
saveButton.onClick=function()
   local titleText=titleEditor.getText().toString()
   local textContent=textEditor.getText().toString()
   if titleText=="" and textContent=="" then
      showToast("Both title and text editor are empty!")
   else
      if titleText~="" then
         table.insert(savedTexts,titleText..": "..textContent)
      else
         table.insert(savedTexts,textContent)
      end
      showToast("Text saved in memory!")
   end
end

shareButton.onClick=function()
   local titleText=titleEditor.getText().toString()
   local textContent=textEditor.getText().toString()
   if titleText=="" and textContent=="" then
      showToast("Both title and text editor are empty!")
   else
      local fullText=(titleText~="" and (titleText.."\n")) or ""..textContent -- Combine title and content for sharing.
      local intent=Intent(Intent.ACTION_SEND)
      intent.setType("text/plain")
      intent.putExtra(Intent.EXTRA_TEXT,fullText)
      activity.startActivity(Intent.createChooser(intent,"Share Text via"))
   end
end

copyButton.onClick=function()
   local text=textEditor.getText()
   if text==nil or text.length()==0 then
      showToast("No text to copy!")
   else
      local clipboard=activity.getSystemService(Context.CLIPBOARD_SERVICE)
      local clip=ClipData.newPlainText("copiedText",text.toString())
      clipboard.setPrimaryClip(clip)
      showToast("Text copied to clipboard!")
   end
end

pasteButton.onClick=function()
   local clipboard=activity.getSystemService(Context.CLIPBOARD_SERVICE)
   local clip=clipboard.getPrimaryClip()
   if clip==nil or clip.getItemCount()==0 then
      showToast("No text to paste!")
   else
      local textToPaste=clip.getItemAt(0).getText()
      if textToPaste==nil then
         showToast("Clipboard is empty!")
      else
         textEditor.setText(textToPaste)
         showToast("Text pasted from clipboard!")
      end
   end
end

cutButton.onClick=function()
   local text=textEditor.getText()
   if text==nil or text.length()==0 then
      showToast("No text to cut!")
   else
      local clipboard=activity.getSystemService(Context.CLIPBOARD_SERVICE)
      clipboard.setPrimaryClip(ClipData.newPlainText("cutText",text.toString()))
      textEditor.setText("")
      showToast("Text cut to clipboard!")
   end  
end

clearButton.onClick=function()
   titleEditor.setText("")  -- Clear title as well.
   textEditor.setText("")
   showToast("All texts cleared!")
end

hindiSpeechButton.onClick=function() startSpeechRecognition("hi-IN") end

englishSpeechButton.onClick=function() startSpeechRecognition("en-US") end

analyzeTextButton.onClick=function() showTextAnalysis() end

showSavedTextsButton.onClick=function() showSavedTexts() end

-- Google Search functionality added here.
googleSearchButton.onClick=function()
    local searchText = textEditor.getText().toString()
    if searchText == "" then
        showToast("Please enter text to search!")
    else
        local intent = Intent(Intent.ACTION_WEB_SEARCH)
        intent.putExtra(SearchManager.QUERY, searchText)  -- Pass the search query.
        if intent.resolveActivity(activity.getPackageManager()) then  -- Check if there's an app that can handle this intent.
            activity.startActivity(intent)  -- Start the search activity.
        else
            showToast("No application available to perform search.")
        end
    end
end